<?php
namespace Teknisa\PLA\Modules\Cardapio\Controllers;

use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DTO\Request\DataSet;
use Zeedhi\Framework\DataSource;

class Pla19000CalculoProdCadenciController {

    protected $environment;
	protected $entityManager;
    protected $connection;
    protected $novoCodigo;

    public function __construct($environment, $entityManager, $connection, $novoCodigo) {
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->connection = $connection;
        $this->novoCodigo = $novoCodigo;
    }

    public function calculoCapacidade(\Zeedhi\Framework\DTO\Request $request, \Zeedhi\Framework\DTO\Response $response) {
        try {
            $row = $request->getRow();
            $retorno = $this->procedureCalculoCapacidade($row);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('calcProdCad', array($retorno)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    private function procedureCalculoCapacidade($row){
        $recalc = isset($row['RECALCULAR']) ? $row['RECALCULAR'] : 'S';
        $procedure = new StoredProcedure($this->connection, 'CALC_CAPACIDADE_PROD_PLA');
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row['CDFILIAL'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERVFILI', Param::PARAM_INPUT, $row['CDSERVFILI'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTCARDAPIO', Param::PARAM_INPUT, $row['DTCARDAPIO'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $this->environment->getNrOrg(), Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_RECALCULAR', Param::PARAM_INPUT, $recalc, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_RETORNO', Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR, 200));
        $response = $procedure->execute();
        return $response['P_RETORNO'];
        
    }
    
    public function verificaCapacidade(\Zeedhi\Framework\DTO\Request $request, \Zeedhi\Framework\DTO\Response $response){
        try {
            $row = $request->getRow();
            $query = "SELECT COUNT(NRPRODCADENCI) QTPRODCAD
                      FROM PRODCADENCI
                      WHERE CDFILIAL =:CDFILIAL
                            AND CDSERVFILI =:CDSERVFILI
                            AND DTCARDAPIO =:DTCARDAPIO";
            $params = array(
                "CDFILIAL" => $row['CDFILIAL'],
                "CDSERVFILI" => $row['CDSERVFILI'],
                "DTCARDAPIO" => $row['DTCARDAPIO']
            );
            $qtProdCad = $this->connection->fetchColumn($query, $params);
            if($qtProdCad == 0) $retorno = $this->procedureCalculoCapacidade($row);
            else $retorno = "CALC_EXISTENTE";
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('calcProdCad', array($retorno)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
       
    }
    
    public function salvaPrato (\Zeedhi\Framework\DTO\Request $request, \Zeedhi\Framework\DTO\Response $response){
        try {
            $this->connection->beginTransaction();
            
            $row = $request->getRow();
            $horaIni = $this->horaToFloat($row['INIPREPPLA']);
            $horaFin = $this->horaToFloat($row['FIMPREPPLA']);
            $row['QUANTIDADE'] = floatval(str_replace(',', '.', $row['QUANTIDADE']));
            $row['QUANTIDADEPLA'] = floatval(str_replace(',', '.', $row['QUANTIDADEPLA']));
            if($row['DURACAO_FLOAT'] == null){
                $row['DURACAO_FLOAT'] = $this->horaToFloat($row['DURACAO']);
            }else $row['DURACAO_FLOAT'] = floatval(str_replace(',', '.',$row['DURACAO_FLOAT']));
            
            $sql = "UPDATE PRODCADENCI 
                    SET INIPREPPLA = :INIPREPPLA,
                        FIMPREPPLA = :FIMPREPPLA,
                        QUANTIDADE = :QUANTIDADE
                    WHERE NRPRODCADENCI = :NRPRODCADENCI ";
            
            $params = array(
                'INIPREPPLA' => $horaIni,
                'FIMPREPPLA' => $horaFin,
                'QUANTIDADE' => $row['QUANTIDADE'],
                'NRPRODCADENCI' => $row['NRPRODCADENCI']
            );
            $this->connection->executeUpdate($sql, $params);
            
            $this->atualizaEquip($horaIni, $row);
            
            if($row['INSERERESTANTE'] == 'S') $this->insereProdRestante($row);
            
            $this->procedureCalcExcedentes($row);
                        
            $this->connection->commit();
        }catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 1));
        }
        
    }
    
    private function atualizaEquip($horaIni, $row){
        $equips = $this->retornaEquipamentos($row, $horaIni);
        foreach($equips as $equip){
            $sql = "UPDATE EQPRODCADENCI 
                    SET TEMPOINI = :TEMPOINI,
                        TEMPOFIN = :TEMPOFIN,
                        QTDEQUIPUTIL = :QTDEQUIPUTIL
                    WHERE NREQPRODCADENCI = :NREQPRODCADENCI";
            
            $params = array(
                'TEMPOINI'   => $horaIni,
                'TEMPOFIN' => $horaIni + floatval($equip['TEMPOPREP']),
                'QTDEQUIPUTIL' => $equip['QTEQUIP'],
                'NREQPRODCADENCI' => $equip['NREQPRODCADENCI']
            );
            $this->connection->executeUpdate($sql, $params);
            $horaIni = $horaIni + floatval($equip['TEMPOPREP']);
        }
    }
    
    private function insereProdRestante($row){
        $row['QUANTIDADE'] = $row['QUANTIDADEPLA'] - $row['QUANTIDADE'];
        $nrprodcad = $this->novoCodigo->getCodeByTableName('PRODCADENCI', $this->environment->getNrOrg());
        $iniPrep = $this->getHorarioProd($row);
        $nrgrupo = $this->getNrgrupo($row);
        $sql = "INSERT INTO PRODCADENCI 
                    (NRPRODCADENCI, CDFILIAL, CDSERVFILI, DTCARDAPIO, 
                     CDPRATO, CDFILAUXPRAT, QUANTIDADE,
                     IDSTATUS, NRCOMENSAIS, INIPREPPLA, FIMPREPPLA, NRGRUPO, 
                     INIPREPREAL, FIMPREPREAL, IDCONFIRMPLAN, IDBLOQPROD, NRORG)
                VALUES 
                    (:NRPRODCADENCI, :CDFILIAL, :CDSERVFILI, :DTCARDAPIO, 
                     :CDPRATO, :CDFILAUXPRAT, :QUANTIDADE,
                     'P', :NRCOMENSAIS, :INIPREPPLA, :FIMPREPPLA, :NRGRUPO,
                     NULL, NULL, 'S', 'N', :NRORG)";
        
        $params = array(
            'NRPRODCADENCI' => $nrprodcad,
            'CDFILIAL'      => $row['CDFILIAL'],
            'CDSERVFILI'    => $row['CDSERVFILI'],
            'DTCARDAPIO'    => $row['DTCARDAPIO'],
            'CDPRATO'       => $row['CDPRATO'],
            'CDFILAUXPRAT'  => $row['CDFILAUXPRAT'],
            'QUANTIDADE'    => $row['QUANTIDADE'],
            'NRCOMENSAIS'   => $row['NRCOMENSAIS'],
            'INIPREPPLA'    => $iniPrep,
            'FIMPREPPLA'    => $iniPrep +  $row['DURACAO_FLOAT'],
            'NRGRUPO'       => $nrgrupo,
            'NRORG'         => $this->environment->getNrOrg()
        );
        $this->connection->prepare($sql)->execute($params);
        
        $this->insereEquipRestante($row, $iniPrep, $nrprodcad);
    }
    
    private function getNrgrupo($row){
        $sql = "SELECT (MAX(NRGRUPO) + 1) NRGRUPO
                FROM PRODCADENCI
                WHERE CDFILIAL = :CDFILIAL
                    AND CDSERVFILI = :CDSERVFILI
                    AND DTCARDAPIO = :DTCARDAPIO
                    AND CDPRATO = :CDPRATO
                    AND CDFILAUXPRAT = :CDFILAUXPRAT
                    AND NRORG =:NRORG";
        
        $params = array(
            'CDFILIAL'      => $row['CDFILIAL'],
            'CDSERVFILI'    => $row['CDSERVFILI'],
            'DTCARDAPIO'    => $row['DTCARDAPIO'],
            'CDPRATO'       => $row['CDPRATO'],
            'CDFILAUXPRAT'  => $row['CDFILAUXPRAT'],
            'NRORG'         => $this->environment->getNrOrg()
        );
        return $this->connection->fetchColumn($sql, $params);
    }
    
    private function insereEquipRestante($row, $iniPrep, $nrprodcad){
        $equips = $this->retornaEquipamentos($row, $iniPrep);
        $nrEqProd = 1;
        foreach($equips as $equip){
            
            $fimPrep = $iniPrep + $equip['TEMPOPREP'];
            
            $sql = "INSERT INTO EQPRODCADENCI 
                        (NREQPRODCADENCI , NRPRODCADENCI , NREQUIPAMENTO , 
                        QTDEQUIPUTIL, TEMPOINI, TEMPOFIN, NRORG)
                    VALUES 
                        ( :NREQPRODCADENCI, :NRPRODCADENCI, :NREQUIPAMENTO, 
                          :QTDEQUIPUTIL, :TEMPOINI, :TEMPOFIN, :NRORG)";
            
            $params = array(
                'NREQPRODCADENCI'   => $nrEqProd,
                'NRPRODCADENCI'     => $nrprodcad,
                'NREQUIPAMENTO'     => $equip['NREQUIPAMENTO'],
                'QTDEQUIPUTIL'      => $equip['QTEQUIP'],
                'TEMPOINI'          => $iniPrep,
                'TEMPOFIN'          => $fimPrep,
                'NRORG'             => $this->environment->getNrOrg()
            );
            $this->connection->prepare($sql)->execute($params);
            
            $iniPrep = $fimPrep;
            $nrEqProd++;
        }
    }
    
    private function getHorarioProd($row){
        $sql = "SELECT MAX(FIMPREPPLA) FIMPREPPLA
                FROM PRODCADENCI
                WHERE CDFILIAL = :CDFILIAL
                    AND CDSERVFILI = :CDSERVFILI
                    AND DTCARDAPIO = :DTCARDAPIO
                    AND CDPRATO = :CDPRATO
                    AND CDFILAUXPRAT = :CDFILAUXPRAT
                    AND NRORG =:NRORG";
        
        $params = array(
            'CDFILIAL'      => $row['CDFILIAL'],
            'CDSERVFILI'    => $row['CDSERVFILI'],
            'DTCARDAPIO'    => $row['DTCARDAPIO'],
            'CDPRATO'       => $row['CDPRATO'],
            'CDFILAUXPRAT'  => $row['CDFILAUXPRAT'],
            'NRORG'         => $this->environment->getNrOrg()
        );
        return $this->connection->fetchColumn($sql, $params);
    }
    
    private function horaToFloat ($horas){
        $result = $horas;
        if(strrpos($horas, ":") != -1){
            $result = 0;
            $hora = substr($horas, 0, strrpos($horas, ":"));
            $min = substr($horas, strrpos($horas, ":")+ 1, strlen($horas));
            $result = round(floatval($hora) + (floatval($min)/60), 6);
        }
        return $result;
    }
    
    private function retornaEquipamentos ($row, $horaIni = 0){
        $sql = "SELECT CALC.*, EQ.NREQPRODCADENCI, (EQ.TEMPOFIN - EQ.TEMPOINI) DURACAO FROM TABLE(CALCULO_EQUIP( :CDFILIAL, :CDPRATO, :CDFILAUXPRAT, :QUANTIDADE, :HRINICSERV, 'INI', :NRORG)) CALC
                LEFT JOIN EQPRODCADENCI EQ ON EQ.NREQUIPAMENTO = CALC.NREQUIPAMENTO AND NRPRODCADENCI = :NRPRODCADENCI";
        
        $params = array(
            'NRPRODCADENCI' => $row['NRPRODCADENCI'],
            'CDFILIAL'      => $row['CDFILIAL'],
            'CDPRATO'       => $row['CDPRATO'],
            'CDFILAUXPRAT'  => $row['CDFILAUXPRAT'],
            'QUANTIDADE'    => $row['QUANTIDADE'],
            'HRINICSERV'    => $horaIni,
            'NRORG'         => $this->environment->getNrOrg()
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function verificaEquip(\Zeedhi\Framework\DTO\Request $request, \Zeedhi\Framework\DTO\Response $response){
        try {
            $row = $request->getRow();
            $query = "SELECT EP.NREQUIPAMENTO, EC.DSEQUIPAMCOZ 
                      FROM EQUIPRATOCOZ EP
                        INNER JOIN EQUIPAMCOZ EC ON EC.NREQUIPAMCOZ = EP.NREQUIPAMENTO
                      WHERE CDCENTRAL = :CDFILIAL
                            AND CDPRATO = :CDPRATO
                            AND CDFILAUXPRAT = :CDFILAUXPRAT";
            $params = array(
                "CDFILIAL" => $row['CDFILIAL'],
                "CDPRATO" => $row['CDPRATO'],
                "CDFILAUXPRAT" => $row['CDFILAUXPRAT']
            );
            $equips = $this->connection->fetchAll($query, $params);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('equipamentos', array($equips)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
       
    }
    
    private function procedureCalcExcedentes($row){
        $procedure = new StoredProcedure($this->connection, 'CALC_EXCEDENTES');
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row['CDFILIAL'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERVFILI', Param::PARAM_INPUT, $row['CDSERVFILI'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTCARDAPIO', Param::PARAM_INPUT, $row['DTCARDAPIO'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRORG', Param::PARAM_INPUT, $this->environment->getNrOrg(), Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
    
}